@@ -3,7 +3,6 @@ from ideas.import_hook import remove_hook
 
 
 def test_transform():
-
     source = """
         switch EXPR:
             case EXPR_1:
@@ -35,16 +34,5 @@ def test_transform():
 
 def test_switch1():
     hook = switch.add_hook()
-
-    try:
-        import switch1  # for testing only this file
-    except ImportError:
-        from . import switch1  # for testing as part of a suite with pytest
-
+    from . import switch1  # noqa
     remove_hook(hook)
-
-
-if __name__ == "__main__":
-    test_transform()
-    test_switch1()
-    print("Done.")
